/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KDCOUTPUTDIALOG_H
#define KDCOUTPUTDIALOG_H

#include "../widgets/kdclabel.h"
#include "../widgets/kdcprogressbar.h"
#include "../lib/kdclib.h"
#include "../widgets/kdcoutputdirstatusbar.h"

#include <QString>
#include <QBasicTimer>
#include <QTimerEvent>
#include <QTime>
#include <QPixmap>
#include <QCloseEvent>
#include <QTreeWidget>
#include <QTreeWidgetItem>

#include <KDialog>
#include <KAction>

class KDCOutputDialog : public KDialog
{
Q_OBJECT
public:
	enum Description { Information, Error, Abort, Done };
	KDCOutputDialog( QWidget *parent = 0 );
	~KDCOutputDialog();
	void setPreviewEnabled( bool enabled );
signals:
	void abort();
	void showPreview();
	void closePreview();
public slots:
	void setDescription( const QString& description, KDCOutputDialog::Description mode = KDCOutputDialog::Information );
	void setJobProgress( int val );
	void setJobPercent( int val );
	void setJobTotalSize( int val );
	void setTotalSize( int val );
	void setJobCount( int val );
	void setJobSpeed( double val );
	void jobsDone();
	void setDiscInfo( const QString& path, quint64 size );
	void abortProcess();
	void uncheckPreview();
	void setCpuUsage( int val );
private slots:
	void on_jobPercentChanged( int, int );
	void stop();
	void setPreviewChecked( bool );
protected:
	void timerEvent( QTimerEvent * );
	void closeEvent( QCloseEvent *event );
private:
	QTreeWidget *p_info;
	KDCProgressBar *p_jobProgress;
	KDCProgressBar *p_jobsProgress;
	QProgressBar *p_cpuUsage;
	KDCOutputDirStatusBar *p_outputDirWidget;

	KDCLabel *p_speed;
	KDCLabel *p_timeInfo;
	QTime *p_elapsedTime;
	QBasicTimer *p_timer;
	KDCLabel *p_outputPath;
	QString p_timeInfoStr;

	KAction *p_previewAction;
	KAction *p_turnOffAction;

	double p_jobSpeed;
	int p_totalSize;
	int p_currentSize;
	int p_previousSize;
	int p_cpu;
};
#endif
