/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcbuilddisccompositejob.h"

#include "../kdcfilefactory.h"
#include "kdcmovieprocesscompositejob.h"
#include "kdcmoviegroupprocesscompositejob.h"
#include "kdcdvdauthorcreatexmlfilejob.h"
#include "kdcvcdxgencreatexmlfilejob.h"
#include "../kdcdvddisc.h"
#include "../kdcxvcddisc.h"
#include "kdcbuilddvddiscstructurejob.h"
#include "kdcbuildxvcddiscstructurejob.h"

#include <QList>
#include <QStringList>

#include <KLocale>

KDCBuildDiscCompositeJob::KDCBuildDiscCompositeJob( KDCDisc *disc, QObject *parent )
	: KDCCompositeJob( KDCCompositeJob::Yes, parent )
{
	p_disc = disc;

	KDCMovieProcessCompositeJob *mJob;
	KDCMovieGroupProcessCompositeJob *mgJob;

	KDCMovie *movie;
	KDCMovieGroup *group;
	p_movies = disc->takeMovies();
	QList<QTreeWidgetItem*>::const_iterator it;
	for( it = p_movies.constBegin(); it != p_movies.constEnd(); ++it )
	{
		if ( ( *it )->type() == KDC::Movie )
		{
			movie = static_cast<KDCMovie*>( *it );
			mJob = new KDCMovieProcessCompositeJob( movie, this );
			connect( mJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
			addSubjob( mJob );
		}
		else
		{
			group = static_cast<KDCMovieGroup*>( *it );
			mgJob = new KDCMovieGroupProcessCompositeJob( group, this );
			connect( mgJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
			addSubjob( mgJob );
		}
	}

	if ( disc->settings()->structureEnabled() )
	{
		if ( disc->discType() == KDC::DVD )
		{
			KDCDVDDisc *dvd = static_cast<KDCDVDDisc*>( disc );
			KDCDVDAuthorCreateXMLFileJob *xJob = new KDCDVDAuthorCreateXMLFileJob( dvd, this );
			connect( xJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
			addSubjob( xJob );

			KDCBuildDVDDiscStructureJob *sJob = new KDCBuildDVDDiscStructureJob( dvd, this );
			connect( sJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
			addSubjob( sJob );
		}
		else
		{
			KDCXVCDDisc *xvcd = static_cast<KDCXVCDDisc*>( disc );
			KDCVCDXGenCreateXMLFileJob *xJob = new KDCVCDXGenCreateXMLFileJob( xvcd, this );
			connect( xJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
			addSubjob( xJob );

			KDCBuildXVCDDiscStructureJob *sJob = new KDCBuildXVCDDiscStructureJob( xvcd, this );
			connect( sJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
			addSubjob( sJob );
		}
	}
}

KDCBuildDiscCompositeJob::~KDCBuildDiscCompositeJob()
{

}

void KDCBuildDiscCompositeJob::start()
{
	emit infoMessage( i18n("Start processing disc %1").arg( p_disc->name() ) );
	emit discInfoMessage( p_disc->settings()->outputPath(), p_disc->size() );
	emit movieChanged( p_movies.takeFirst() );
	if ( !KDCFileFactory::createDirectory( KDCFileFactory::discOutputDirectory( p_disc ) ) )
	{
		setError( i18n("There was an error during creating output directory.") );
		emit finished( this );
	}
	else
		firstSubjob()->start();
}

void KDCBuildDiscCompositeJob::on_jobFinished( KDCJob *job )
{
	if ( job->hasErrors() )
	{
		if ( job->error() != "0" )
			emit errorMessage( job->error() );
		removeSubjob( job );
		setError( "0" );
		emit finished( this );
	}
	else
	{
		removeSubjob( job );
		if ( hasSubjobs() )
		{
			if ( !p_movies.isEmpty() )
				emit movieChanged( p_movies.takeFirst() );
			else
				emit movieChanged( NULL );
			firstSubjob()->start();
		}
		else
		{
			emit infoMessage( i18n("Processing disc %1 sucesfully finished").arg( p_disc->name() ) );
			emit finished( this );
		}
	}
}
