/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcconvertframejob.h"

#include "../kdcfilefactory.h"

#include <QStringList>

#include <KLocale>

KDCConvertFrameJob::KDCConvertFrameJob(	const QString& workDir, const QString& inputFile,
																				const QString& outputFile, int scaleX, int scaleY, QObject *parent )
	: KDCJob( parent )
{
	p_workDir = workDir;
	p_inputFile = inputFile;

	p_process = new KProcess( this );
	connect( p_process, SIGNAL( finished( int , QProcess::ExitStatus ) ), this, SLOT( on_finished( int , QProcess::ExitStatus ) ) );
	p_process->setWorkingDirectory( p_workDir );
	p_process->setOutputChannelMode( KProcess::MergedChannels );
	connect( p_process, SIGNAL( readyReadStandardOutput() ), this, SLOT( on_readyReadStandardOutput() ) );

	QString scale = QString( "%1x%2\\!" ).arg( scaleX ).arg( scaleY );
	QStringList arg;
	arg << "convert" << "-resize" << scale << inputFile << "-thumbnail" << "184x106" << "-bordercolor" << "black";
	arg << "-border" << "80" << "-gravity" << "center" << "-crop" << "184x106+0+0" << "+repage" << outputFile;

	p_process->setShellCommand( arg.join(" ").toUtf8() );
	qDebug( arg.join( " " ).toUtf8() );
	setTotalSize( 100 );
}

KDCConvertFrameJob::~KDCConvertFrameJob()
{
	if ( p_process )
	{
		p_process->disconnect();
		if ( p_process->state() == QProcess::Running )
			p_process->close();
		delete p_process;
	}
}

void KDCConvertFrameJob::on_readyReadStandardOutput()
{
	QString error( p_process->readAllStandardOutput() );
	setFlag( 1, TRUE );
	setError( error );
	p_process->close();
}

void KDCConvertFrameJob::start()
{
	emit percent( 0 );
	emit infoMessage( i18n("Converting frame...") );
	p_process->start();
}

void KDCConvertFrameJob::on_finished( int code , QProcess::ExitStatus status )
{
	qDebug( "KDCConvertFrameJob::finished()::Codec = %d, status = %d", code, status );
	if ( status == QProcess::CrashExit && !flag( 1 ) )
		setError( i18n("There was an error during process. Error code is %1").arg( code ) );
	else
		emit percent( 100 );

	KDCFileFactory::removeFile( p_workDir + '/' + p_inputFile );
	emit finished( this );
}
