/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcmovieconvertjob.h"

#include "../kdcaudiostream.h"
#include "../kdcvideostream.h"
#include "../kdcconfig.h"
#include "../lib/kdclib.h"
#include "../kdcdisc.h"
#include "../kdcfilefactory.h"

#include <QByteArray>

#include <KLocale>

KDCMovieConvertJob::KDCMovieConvertJob( const KDCMovieGroup *group, QObject *parent )
	: KDCJob( parent )
{
	qDebug( "KDCMovieConvertJob::group::start" );
	QStringList arg = prepareMovie( group->movie( 0 ) );
	int size = 0;
	p_fps = group->movie( 0 )->fps();
	arg.append( KDCFileFactory::movieName( group ) );
	KDCMovie *movie;
	for ( int i=0; i<group->count(); ++i )
	{
		movie = group->movie( i );
		arg.append( movie->fileName() );
		size += qRound( movie->videoStream()->metaData()->length()/1000.0 * p_fps );
		p_files.insert( size, KDCFileFactory::fileName( movie->fileName() ) );
	}

	p_process->setProgram( "nice", arg );
	qDebug( QString( "KDCMovieConvertJob:: " + arg.join(" ") ).toUtf8() );
	setTotalSize( size );
	qDebug( "KDCMovieConvertJob::group::end" );
}

KDCMovieConvertJob::KDCMovieConvertJob( const KDCMovie *movie, QObject *parent )
	: KDCJob( parent )
{
	qDebug( "KDCMovieConvertJob::movie::start" );
	QStringList arg = prepareMovie( movie );
	arg.append( KDCFileFactory::movieName( movie ) );
	arg.append( movie->fileName() );
	p_fps = movie->fps();
	setTotalSize( qRound( movie->videoStream()->metaData()->length()/1000.0 * p_fps ) );
	p_files.insert( totalSize(), KDCFileFactory::fileName( movie->fileName() ) );

	p_process->setProgram( "nice", arg );
	qDebug( QString( "KDCMovieConvertJob:: " + arg.join(" ") ).toUtf8() );
	
	qDebug( "KDCMovieConvertJob::movie::start" );
}

QStringList KDCMovieConvertJob::writeHeader( KDC::DiscType type, double outFps )
{
	qDebug( "KDCMovieConvertJob::writeHeader()::begin" );
	QStringList list;
	list.append( "-of" );
	list.append( "mpeg" );
	list.append( "-mpegopts" );
	if ( type == KDC::DVD )
	{
		if ( outFps == 23.976 )
			list.append( "format=dvd:tsaf:telecine" );
		else
			list.append( "format=dvd:tsaf" );
	}
	else
		if ( type == KDC::SVCD || type == KDC::CVD )
			list.append( "format=xsvcd" );
	else
		list.append( "format=xvcd" );

	qDebug( "KDCMovieConvertJob::writeHeader()::end" );
	return list;
}

QStringList KDCMovieConvertJob::prepareMovie( const KDCMovie *movie )
{
	qDebug( "KDCMovieConvertJob::prepareMovie::start" );
	p_copyA = FALSE;
	p_copyV = FALSE;
	p_process = new KProcess( this );
	p_process->setOutputChannelMode( KProcess::MergedChannels );
	connect( p_process, SIGNAL( readyReadStandardOutput() ), this, SLOT( on_readyReadStandardOutput() ) );
	connect( p_process, SIGNAL( finished( int , QProcess::ExitStatus ) ), this, SLOT( on_finished( int , QProcess::ExitStatus ) ) );

	QStringList arguments;
	QString audioOptions;
	QString videoOptions;
	KDCAudioStream *aStream = NULL;
	KDCDisc *disc = movie->disc();
	int index = 0;
	if ( movie->audioStreamGroup() )
	{
		for( int i=0; i<movie->audioStreamGroup()->count(); ++i )
			if ( movie->audioStreamGroup()->stream( i )->enabledMode() == KDC::Enabled )
			{
				aStream = movie->audioStreamGroup()->stream( i );
				index = i;
				break;
			}
	}
	KDCVideoStream *vStream = movie->videoStream();

	arguments.append( "-n" );
	arguments.append( QString::number( KDCConfig::niceLevel() ) );
	arguments.append( "mencoder" );
	qDebug( "KDCMovieConvertJob::prepareMovie::audio" );
	if ( aStream )
	{
		if ( index > 0 )
		{
			arguments.append( "-aid" );
			arguments.append( QString::number( index + 1 ) );
		}
		arguments.append( "-oac" );
		if ( aStream->mode() == KDC::Copy )
		{
			p_copyA = TRUE;
			arguments.append( "copy" );
			if ( aStream->delay() != 0 )
			{
				arguments.append( "-audio-delay" );
				arguments.append( QString::number( aStream->delay() ) );
			}
		}
		else
		{
			arguments.append( "lavc" );
			if ( disc->discType() == KDC::DVD )
			{
				if ( aStream->metaData()->sampleRate() != 48000 )
				{
					arguments.append( "-srate" );
					arguments.append( "48000" );
					arguments.append( "-af" );
					arguments.append( "lavcresample=48000" );
				}
			}
			else // SVCD, VCD, CVD
			{
				if ( aStream->metaData()->sampleRate() != 44100 )
				{
					arguments.append( "-srate" );
					arguments.append( "44100" );
					arguments.append( "-af" );
					arguments.append( "lavcresample=44100" );
				}
			}
			if ( aStream->delay() != 0 )
			{
				arguments.append( "-audio-delay" );
				arguments.append( QString::number( aStream->delay() ) );
			}
			if ( aStream->codec() == KDC::AC3 )
				audioOptions.append( "acodec=ac3" );
			else
				audioOptions.append( "acodec=mp2" );
			arguments.append( "-channels" );
			arguments.append( QString::number( aStream->channelCount() ) );
			audioOptions.append( ":abitrate=" );
			audioOptions.append( QString::number( aStream->bitrate() ) );
		}
	}
	else
		arguments.append( "-nosound" );

	qDebug( "KDCMovieConvertJob::prepareMovie::video" );
	arguments.append( "-ovc" );
	if ( vStream->mode() == KDC::Copy )
	{
		p_copyV = TRUE;
		arguments.append( "copy" );
		arguments << writeHeader( disc->discType(), movie->fps() );

		if ( aStream )
		{
			if ( aStream->mode() == KDC::Resample )
				arguments.append( "-lavcopts" );
		}
	}
	else
	{
		arguments.append( "lavc" );
		arguments << writeHeader( disc->discType(), movie->fps() );

		Expand expand = computeExpand( movie );
		if (	vStream->metaData()->width() != vStream->width() || vStream->metaData()->height() != vStream->height() ||
					movie->movieGroup() )
		{
			QString tmp;
			arguments.append( "-vf" );
			if ( expand.upX != 0 || expand.upY != 0 )
			{
				tmp.append( "expand=" );
				tmp.append( QString::number( expand.addX ) );
				tmp.append( ":" );
				tmp.append( QString::number( expand.addY ) );
				tmp.append( ":" );
				tmp.append( QString::number( expand.upX ) );
				tmp.append( ":" );
				tmp.append( QString::number( expand.upY ) );
				tmp.append( "," );
			}
			tmp.append( "scale=" );
			tmp.append( QString::number( vStream->width() ) );
			tmp.append( ":" );
			tmp.append( QString::number( vStream->height() ) );
			tmp.append( ",harddup" );
			arguments.append( tmp );
		}
		arguments.append( "-lavcopts" );
		videoOptions.append( "vcodec=" );
		if ( disc->discType() == KDC::DVD && vStream->codec() == KDC::mpeg2 )
		{
			videoOptions.append( "mpeg2video" );	
			videoOptions.append( ":vrc_buf_size=1835" );
			videoOptions.append( ":vrc_maxrate=" + QString::number( qMin( vStream->bitrate() + 1000, 9800 ) ) );
			videoOptions.append( ":vstrict=0" );
		}
		else
			if ( disc->discType() == KDC::DVD && vStream->codec() == KDC::mpeg1 )
			{
				videoOptions.append( "mpeg1video" );	
				videoOptions.append( ":vrc_buf_size=1835" );
				videoOptions.append( ":vrc_maxrate=" + QString::number( qMin( vStream->bitrate() + 500, 1856 ) ) );
				videoOptions.append( ":vstrict=0" );
			}
		else
			if ( disc->discType() == KDC::SVCD || disc->discType() == KDC::CVD )
			{
				videoOptions.append( "mpeg2video" );	
				videoOptions.append( ":vrc_buf_size=917" );
				videoOptions.append( ":vrc_maxrate=2600" );
			}
		else
		{
			videoOptions.append( "mpeg1video" );	
			videoOptions.append( ":vrc_buf_size=327" );
			videoOptions.append( ":vrc_maxrate=1152" );
		}
		if ( vStream->hqDVDMode() == KDC::Enabled )
			videoOptions.append( ":trell:mbd=2:precmp=2:subcmp=2:cmp=2:dia=-10:predia=-10:cbp:mv0:vqmin=1:lmin=1:dc=10" );
		else
		{
			if ( vStream->trellisMode() == KDC::Enabled )
				videoOptions.append( ":trell" );
			videoOptions.append( ":mbd=" + QString::number( vStream->macroBlockMode() ) );
		}
		if ( vStream->deinterlaceMode() == KDC::Linear )
			videoOptions.append( ":pp=lb" );
		else
			if ( vStream->deinterlaceMode() == KDC::Median )
				videoOptions.append( ":pp=md" );
		else
			if ( vStream->deinterlaceMode() == KDC::Linear )
				videoOptions.append( ":pp=fd" );
		if ( vStream->closestGOPMode() == KDC::Enabled )
			videoOptions.append( ":cgop:sc_threshold=1000000000" );
		videoOptions.append( ":vbitrate=" + QString::number( vStream->bitrate() ) );
		if ( movie->videoFormat() == KDC::NTSC )
			videoOptions.append( ":keyint=18" );
		else
			videoOptions.append( ":keyint=15" );

		if ( expand.ar == KDC::LetterBox )
			videoOptions.append( ":aspect=4/3" );
		else
			videoOptions.append( ":aspect=16/9" );
	}
	if ( videoOptions.isEmpty() && !audioOptions.isEmpty() )
		arguments.append( audioOptions );
	else
		if ( !videoOptions.isEmpty() && audioOptions.isEmpty() )
			arguments.append( videoOptions );
	else
		if ( !videoOptions.isEmpty() && !audioOptions.isEmpty() )
		 arguments.append( videoOptions + ":" + audioOptions );
	if ( !p_copyV )
	{
		arguments.append( "-ofps" );
		if ( movie->fps() == 29.970 )
			arguments.append( "30000/1001" );
		else
			if ( movie->fps() == 23.976 )
				arguments.append( "24000/1001" );
		else
			arguments.append( "25" );
	}
	arguments.append( "-o" );
	qDebug( "KDCMovieConvertJob::prepareMovie::end" );
	return arguments;
}

KDCMovieConvertJob::~KDCMovieConvertJob()
{
	if ( p_process )
	{
		p_process->disconnect();
		if ( p_process->state() == QProcess::Running )
			p_process->close();
		delete p_process;
	}
	qDebug( "~KDCMovieConvertJob" );
}

void KDCMovieConvertJob::start()
{
	qDebug( "KDCMovieConvertJob::start()" );
	emit jobMainProgram( "mencoder" );
	emit jobTotalSize( totalSize() );
	emit percent( 0 );
	p_it = p_files.constBegin();
	if ( p_copyA && p_copyV )
		emit infoMessage( i18n("Start writing header in file ") + p_it.value() );
	else
		emit infoMessage( i18n("Start converting file ") + p_it.value() );
	p_process->start();
	p_timerID = startTimer( KDCConfig::updateData() );
	qDebug( p_process->program().join(" ").toUtf8() );
}

void KDCMovieConvertJob::on_readyReadStandardOutput()
{
/*
	QByteArray data( p_process->readAllStandardOutput() );
	if ( data.lastIndexOf( "Pos", 3 ) != -1 )
	{
		QByteArray temp;
		int idx = data.indexOf( 's', 9 ) - 1;
		char byte = data.at( idx );
		while ( byte != ' ' )
		{
			temp.prepend( byte );
			--idx;
			byte = data.at( idx );
		}
		double pos = temp.toDouble();
		//emit progress( pos );
		//idx = data.indexOf( 's', 7 ) + 1;
		//int tmp = data.indexOf( 'f', idx ); 
		//temp = data.mid( idx, tmp - idx ).trimmed();
		//int frame = temp.toInt();
		int frame = qRound( pos*p_fps );
		emit progress( frame );
		temp.clear();
		idx = data.indexOf( '%', 22 ) + 2; //tmp + 5
		byte = data.at( idx );
		while ( byte != 'f' )
		{
			temp.append( byte );
			++idx;
			byte = data.at( idx );
		}
		temp.trimmed();
		emit speed( temp.toDouble() );

		if ( ( pos > p_it.key() ) && ( p_it != p_files.constEnd() ) )
		{
			++p_it;
			if ( p_it != p_files.constEnd() )
			{
				if ( p_copyA && p_copyV )
					emit infoMessage( i18n("Start writing header in file ") + p_it.value() );
				else
					emit infoMessage( i18n("Start converting file ") + p_it.value() );
			}
		}
	}
	else
	{
		int idx = data.indexOf( "FATAL" );
		if ( idx != -1 )
		{
			setError( QString( data.mid( idx ) ) );
			setFlag( 1, TRUE );
			p_process->close();
		}
	}
*/
}

void KDCMovieConvertJob::on_finished( int code, QProcess::ExitStatus status )
{
	killTimer( p_timerID );
	qDebug( "KDCMovieConvertJob::end:: code = %d, status = %d", code, status );
	if ( ( status == QProcess::CrashExit || code != 0 ) && !flag( 1 ) )
		setError( i18n("There was an error during prosess. Error code is %1").arg( code ) );
	else
	{
		emit percent( 100 );
		if ( p_copyA && p_copyV )
			emit infoMessage( i18n("Writing header sucesfully finished") );
		else
			emit infoMessage( i18n("Convert sucesfully finished") );
	}
	qDebug( "KDCMovieConvertJob::emit finished()" );
	emit finished( this );
}

Expand KDCMovieConvertJob::computeExpand( const KDCMovie *movie )
{
	// This method is from DeVeDe program + my ideas.
	Expand res;
	int width;
	int height;
	KDCVideoStreamMetaData *video = movie->videoStream()->metaData();
	if ( ( video->width() % 2 ) == 1 )
		width = video->width() + 1;
	else
		width = video->width();
	if ( ( video->height() % 2 ) == 1 )
		height = video->height() + 1;
	else
		height = video->height();

	res.addX = width;
	if ( movie->videoStream()->aspectRatio() == KDC::WideScreen )
	{
		res.ar = KDC::WideScreen;
		res.addY = qRound( height*video->aspectRatio()/1.778 );
	}
	else
	{
		res.ar = KDC::LetterBox;
		res.addY = qRound( height*video->aspectRatio()/1.333 );
	}

	if ( ( res.addY % 2 ) == 1 )
		res.addY++;

	if ( res.addY < ( video->height() + 5 ) )
	{
		res.addX = video->width();
		res.addY = video->height();
		res.upX = 0;
		res.upY = 0;
	}
	else
	{
		res.upX = 0;
		res.upY = qRound( 0.5*( res.addY - video->height() ) );
		if ( ( res.upY % 2 ) == 1 )
			res.upY++;
	}

	qDebug( "KDCMovieConvertJob::computeExpand()::end" );
	return res;
}

void KDCMovieConvertJob::setOutputFile( const QString& file )
{
	QStringList arg = p_process->program();
	arg.removeFirst();
	arg[ arg.indexOf( "-o" ) + 1 ] = file;
	p_process->setProgram( "nice", arg );
}

void KDCMovieConvertJob::timerEvent( QTimerEvent *event )
{
	if ( event->timerId() == p_timerID )
	{
		QByteArray data( p_process->readAllStandardOutput() );
		if ( data.lastIndexOf( "Pos", 3 ) != -1 )
		{
			QByteArray temp;
			//int idx = data.indexOf( 's', 9 ) - 1;
			int idx = data.indexOf( '.', 7 ) - 1;
			char byte = data.at( idx );
			while ( byte != ' ' )
			{
				temp.prepend( byte );
				--idx;
				byte = data.at( idx );
			}
			int pos = temp.toInt();
			int frame = qRound( pos*p_fps );
			emit progress( frame );
			temp.clear();
			idx = data.indexOf( '%', 22 ) + 2; //tmp + 5
			byte = data.at( idx );
			while ( byte != 'f' )
			{
				temp.append( byte );
				++idx;
				byte = data.at( idx );
			}
			temp.trimmed();
			emit speed( temp.toDouble() );

			if ( ( pos > p_it.key() ) && ( p_it != p_files.constEnd() ) )
			{
				++p_it;
				if ( p_it != p_files.constEnd() )
				{
					if ( p_copyA && p_copyV )
						emit infoMessage( i18n("Start writing header in file ") + p_it.value() );
					else
						emit infoMessage( i18n("Start converting file ") + p_it.value() );
				}
			}
		}
		else
		{
			int idx = data.indexOf( "FATAL" );
			if ( idx != -1 )
			{
				setError( QString( data.mid( idx ) ) );
				setFlag( 1, TRUE );
				p_process->close();
			}
		}
	}
}
