/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcreadfileinfosubjob.h"

#include <QStringList>

#include <KLocale>

KDCReadFileInfoSubjob::KDCReadFileInfoSubjob( QObject *parent )
	:	KDCJob( parent )
{
	p_process = new KProcess( this );
	p_process->setOutputChannelMode( KProcess::MergedChannels );
	connect( p_process, SIGNAL( readyReadStandardOutput() ), this, SLOT( on_readyReadStandardOutput() ) );
	connect( p_process, SIGNAL( finished( int, QProcess::ExitStatus ) ), this, SLOT( on_finished( int, QProcess::ExitStatus ) ) );
}

KDCReadFileInfoSubjob::~KDCReadFileInfoSubjob()
{
	if ( p_process )
	{
		p_process->disconnect();
		if ( p_process->state() == QProcess::Running )
			p_process->close();
		delete p_process;
	}
}

void KDCReadFileInfoSubjob::on_readyReadStandardOutput()
{
	//TODO
	p_data.append( p_process->readAllStandardOutput() );
}

void KDCReadFileInfoSubjob::start()
{

}

void KDCReadFileInfoSubjob::start( KDCReadFileInfoSubjob::Mode mode, int id )
{
	p_mode = mode;
	p_id = id;
	p_data.clear();
	if ( mode == AudioVideo )
		p_process->setProgram( "grep", QStringList() << "ID_" );
	else
		p_process->setProgram( "grep", QStringList() << "ID_AUDIO_" );
	p_process->start();
}

KProcess* KDCReadFileInfoSubjob::jobProcess() const
{
	return p_process;
}

void KDCReadFileInfoSubjob::on_finished( int exitCode, QProcess::ExitStatus exitStatus )
{
	qDebug( "KDCReadFileInfoSubjob::finished()::codec = %d, status = %d", exitCode, exitStatus );
	if ( exitStatus == QProcess::CrashExit && !flag( 1 ) )
		setError( i18n("There was an error during process. Error codec is %1").arg( exitCode ) );
	else
		if ( !p_data.isEmpty() )
		{
			if ( p_mode == AudioVideo )
			{
				int idx = p_data.indexOf( "ID_VIDEO_ID=" );
				if ( idx != -1  ) //ther's video stream
				{
					QList<int> listID;
					bool tmp = TRUE;
					idx = 0;
					QByteArray temp;
					char byte;
					while ( tmp )
					{
						idx = p_data.indexOf( "ID_AUDIO_ID=", idx );
						if ( idx != -1 )
						{
							idx += 12;
							byte = p_data.at( idx );
							while ( byte != '\n' )
							{
								temp.append( byte );
								++idx;
								byte = p_data.at( idx );
							}
							listID.append( temp.toInt() );
							temp.clear();
						}
						else
							tmp = FALSE;
					}
					int idx = p_data.indexOf( "ID_DEMUXER=" ) + 11;
					temp.clear();
					byte = p_data.at( idx );
					while ( byte != '\n' )
					{
						temp.append( byte );
						++idx;
						byte = p_data.at( idx );
					}
					QString demuxer = QString( temp );

					temp.clear();
					idx += 17;
					byte = p_data.at( idx );
					while ( byte != '\n' )
					{
						temp.append( byte );
						++idx;
						byte = p_data.at( idx );
					}
					QString formatV = QString( temp );

					temp.clear();
					idx += 18;
					byte = p_data.at( idx );
					while ( byte != '\n' )
					{
						temp.append( byte );
						++idx;
						byte = p_data.at( idx );
					}
					int bitrateV = qRound( temp.toInt() / 1000.0 );

					temp.clear();
					idx += 16;
					byte = p_data.at( idx );
					while ( byte != '\n' )
					{
						temp.append( byte );
						++idx;
						byte = p_data.at( idx );
					}
					int width = temp.toInt();

					temp.clear();
					idx += 17;
					byte = p_data.at( idx );
					while ( byte != '\n' )
					{
						temp.append( byte );
						++idx;
						byte = p_data.at( idx );
					}
					int height = temp.toInt();

					temp.clear();
					idx += 14;
					byte = p_data.at( idx );
					while ( byte != '\n' )
					{
						temp.append( byte );
						++idx;
						byte = p_data.at( idx );
					}
					double fps = temp.toDouble();

					QString formatA;
					if ( !listID.isEmpty() )
					{
						temp.clear();
						idx = p_data.indexOf( "AT=", idx + 32 ) + 3;
						byte = p_data.at( idx );
						while ( byte != '\n' )
						{
							temp.append( byte );
							++idx;
							byte = p_data.at( idx );
						}
						formatA = QString( temp );
						idx = p_data.indexOf( "TH=", idx + 58 ) + 3;
					}
					else
					idx = p_data.indexOf( "TH=", idx + 26 ) + 3;

					temp.clear();
					byte = p_data.at( idx );
					while ( byte != '\n' )
					{
						temp.append( byte );
						++idx;
						byte = p_data.at( idx );
					}
					double length = temp.toDouble();

					if ( !listID.isEmpty() )
					{
						temp.clear();
						idx = p_data.indexOf( "E=", idx + 33 ) + 2;
						byte = p_data.at( idx );
						while ( byte != '\n' )
						{
							temp.append( byte );
							++idx;
							byte = p_data.at( idx );
						}
						int bitrateA = qRound( temp.toInt() / 1000 );

						temp.clear();
						idx += 15;
						byte = p_data.at( idx );
						while ( byte != '\n' )
						{
							temp.append( byte );
							++idx;
							byte = p_data.at( idx );
						}
						int sampleRate = temp.toInt();

						temp.clear();
						idx += 14;
						byte = p_data.at( idx );
						while ( byte != '\n' )
						{
							temp.append( byte );
							++idx;
							byte = p_data.at( idx );
						}
						int channels = temp.toInt();

						emit foundAudioStream( new KDCAudioStreamMetaData( listID.first(), formatA, channels, sampleRate, bitrateA ) );
					}
					temp.clear();
					idx = p_data.indexOf( "T=", idx ); //+14
					double aspect;
					if ( idx != -1 )
					{
						idx += 2;
						byte = p_data.at( idx );
						while ( byte != '\n' )
						{
							temp.append( byte );
							++idx;
							byte = p_data.at( idx );
						}
						aspect = temp.toDouble();
					}
					else
						aspect = (double)width/(double)(height);
					emit foundVideoStream( new KDCVideoStreamMetaData( demuxer, formatV, width, height, bitrateV, fps, length, aspect ) );
					emit foundAudioStreamID( listID );
				} //is there video stream
			}
			else // only audio
			{
				QByteArray temp;
				int idx = p_data.indexOf( "AT=", 41 ) + 3;
				char byte = p_data.at( idx );
				while ( byte != '\n' )
				{
					temp.append( byte );
					++idx;
					byte = p_data.at( idx );
				}
				QString format = QString( temp );

				temp.clear();
				idx = p_data.indexOf( "E=", idx + 66 ) + 2;
				byte = p_data.at( idx );
				while ( byte != '\n' )
				{
					temp.append( byte );
					++idx;
					byte = p_data.at( idx );
				}
				int bitrate = qRound( temp.toInt() / 1000.0 );

				temp.clear();
				idx += 15;
				byte = p_data.at( idx );
				while ( byte != '\n' )
				{
					temp.append( byte );
					++idx;
					byte = p_data.at( idx );
				}
				int sampleRate = temp.toInt();

				temp.clear();
				idx += 14;
				byte = p_data.at( idx );
				while ( byte != '\n' )
				{
					temp.append( byte );
					++idx;
					byte = p_data.at( idx );
				}
				int channels = temp.toInt();

				emit foundAudioStream( new KDCAudioStreamMetaData( p_id, format, channels, sampleRate, bitrate ) );
			}
		}
	emit finished( this );
}
