/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcaudiostreamgroup.h"

KDCAudioStreamGroup::KDCAudioStreamGroup()
{

}

KDCAudioStreamGroup::KDCAudioStreamGroup( KDCAudioStreamGroup *stream )
{
	for ( int i=0; i<stream->count(); ++i )
		p_audio.append( stream->stream( i ) );
}

KDCAudioStreamGroup::~KDCAudioStreamGroup()
{
	while ( !p_audio.isEmpty() )
		delete p_audio.takeFirst();
}

KDCAudioStream* KDCAudioStreamGroup::stream( int index ) const
{
	if ( index < p_audio.size() )
		return p_audio.at( index );
	else
		return NULL;
}

int KDCAudioStreamGroup::count() const
{
	return p_audio.count();
}


void KDCAudioStreamGroup::addStream( KDCAudioStream *stream )
{
	p_audio.append( stream );
}

KDCAudioStream* KDCAudioStreamGroup::removeStream( int index )
{
	if ( index < p_audio.size() )
		return p_audio.takeAt( index );
	else
		return NULL;
}

void KDCAudioStreamGroup::copySettings( const KDCAudioStreamGroup *group )
{
	QList<KDCAudioStream*>::iterator it;
	int index = 0;
	for ( it = p_audio.begin(); it != p_audio.end(); ++it )
	{
		( *it )->copySettings( group->stream( index ) );
		++index;
	}
}

bool KDCAudioStreamGroup::operator!=( const KDCAudioStreamGroup& group )
{
	if ( this && &group )
	{
		if ( count() != group.count() )
			return TRUE;
		for ( int i=0; i<count(); ++i )
			if ( *stream( i ) != *group.stream( i ) )
				return TRUE;
	}
	else
		if ( ( this != NULL && &group == NULL ) || ( this == NULL && &group != NULL ) )
			return TRUE;

	return FALSE;
}
