/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdclib.h"


int KDCLib::isNull( void *ptr )
{
	if ( ptr )
		return 1;
	else
		return 0;
}

const QString KDCLib::audioCodecName( const QString& codec )
{
	if ( codec == "8192" )
		return "ac3";
	else
		if ( codec == "80" )
			return "mp2";
	else
		if ( codec == "85" )
			return "mp3";
	else
		return codec;
}

const KDC::AudioCodec KDCLib::audioCodec( const QString& codec )
{
	if ( codec == "8192" )
		return KDC::AC3;
	else
		if ( codec == "80" )
			return KDC::MP2;
	else
		return KDC::IncorrectAudioCodec;
}

const QString KDCLib::videoCodecName( const QString& codec )
{
	if ( codec == "0x10000001" )
		return "MPEG-1";
	else
		if ( codec == "0x10000002" )
			return "MPEG-2";
	else
		return codec;
}

const KDC::VideoCodec KDCLib::videoCodec( const QString& codec )
{
	if ( codec == "0x10000001" )
		return KDC::mpeg1;
	else
		if ( codec == "0x10000002" )
			return KDC::mpeg2;
	else
		return KDC::IncorrectVideoCodec;
}

const KDC::AspectRatio KDCLib::aspectRatio( const KDCMovie *movie )
{
	if (	isInRange( movie->videoStream()->metaData()->aspectRatio(), 1.76, 1.8 ) &&
				movie->disc()->discType() == KDC::DVD &&
				( ( movie->videoStream()->codec() == KDC::mpeg2 && movie->videoStream()->width() == 720 &&
					( movie->videoStream()->height() == 576 || movie->videoStream()->height() == 480 ) ) ||
					( movie->videoStream()->codec() == KDC::mpeg1 && movie->videoStream()->width() == 352 &&
					( movie->videoStream()->height() == 288 || movie->videoStream()->height() == 240 ) ) ) )
		return KDC::WideScreen;
	else
		return KDC::LetterBox;
}

bool KDCLib::isInRange( int value, int min, int max )
{
	if ( ( value <= max ) && ( value >= min ) )
		return TRUE;
	else
		return FALSE;
}

bool KDCLib::isInRange( double value, double min, double max )
{
	if ( ( value <= max ) && ( value >= min ) )
		return TRUE;
	else
		return FALSE;
}


int KDCLib::getCorrectHeight( int width, int oldHeight, KDC::VideoFormat format )
{
	if ( format == KDC::PAL )
	{
		if ( width == 352 || ( oldHeight - 240 ) == 0 )
			return 288;
		return 576;
	}
	else
	{
		if ( width == 352 || ( oldHeight - 288 ) == 0 )
			return 240;
		return 480;
	}
}
