/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mainwindow.h"

#include "kdcconfig.h"
#include "kdcdisc.h"
#include "kdcsvcddisc.h"
#include "kdcvcddisc.h"
#include "kdccvddisc.h"
#include "kdcmovie.h"
#include "kdcnamespace.h"
#include "widgets/kdclabel.h"
#include "dialogs/kdcconfig/kdcconfigdialog.h"
#include "lib/kdclib.h"
#include "widgets/kdcmainwindowframe.h"
#include "jobs/kdcreadfilecompositejob.h"
#include "jobs/kdcbuildallcompositejob.h"
#include "jobs/kdconflypreviewjob.h"
#include "jobs/kdcstaticpreviewjob.h"
#include "kdcfilefactory.h"

#include <QWidget>
#include <QKeySequence>
#include <QDebug>

#include <KFileDialog>
#include <KUrl>
#include <KLocale>
#include <KMessageBox>
#include <KActionCollection>
#include <KMenu>
#include <KApplication>
#include <KConfig>
#include <KConfigGroup>
#include <KStandardAction>
#include <KXMLGUIFactory>
#include <KStandardShortcut>
#include <KStatusBar>

MainWindow::MainWindow( QWidget *parent )
 : KXmlGuiWindow( parent )
{
	discFactory = new KDCDiscFactory;
	KDCMainWindowFrame *frame = new KDCMainWindowFrame( discFactory );

	connect( discFactory, SIGNAL( currentDiscContextMenu( const QPoint & ) ), this, SLOT( on_currentDiscContextMenu( const QPoint & ) ) );
	connect( discFactory, SIGNAL( currentObjectChanged( QTreeWidgetItem * ) ), this, SLOT( on_currentObjectChanged( QTreeWidgetItem * ) ) );
	connect( discFactory, SIGNAL( canMoveUpDown( bool, bool ) ), this, SLOT( on_canMoveUpDown( bool, bool ) ) );
	connect( discFactory, SIGNAL( currentDiscChanged( KDCDisc * ) ), this, SLOT( on_currentDiscChanged( KDCDisc * ) ) );
	connect( discFactory, SIGNAL( currentDiscSettingsChanged( KDC::MediaSize , quint64 , const QString& ) ), this, SLOT( on_currentDiscSettingsChanged( KDC::MediaSize , quint64 , const QString& ) ) );
	connect( discFactory, SIGNAL( needToOpenSomeFiles( const QStringList& ) ), this, SLOT( openFiles( const QStringList& ) ) );
	connect ( discFactory, SIGNAL( needToCloseDisc( KDCDisc * ) ), this, SLOT( on_needToCloseDisc( KDCDisc * ) ) );

	setCentralWidget( frame );

	createStatusBar();
	createActions();
	addActionsToCollection();
	setupGUI();
	stateChanged( "apps_clear" );
	setWindowIcon( KDCFileFactory::appIcon() );

	outputDirStatus->start();
	outputDirStatus->setOutputPath( KDCConfig::defaultOutputPath() );
}

MainWindow::~MainWindow()
{
	int i;
	for( i=0; i<groupActions.size(); ++i )
	{
		groupActions.at( i )->disconnect();
		addToGroupAction->remove( groupActions.at( i ) );
		delete groupActions.at( i );
	}

	for( i=0; i<movieActions.size(); ++i )
	{
		movieActions.at( i )->disconnect();;
		createGroupAction->remove( movieActions.at( i ) );
		delete movieActions.at( i );
	}

	KDCFileFactory::clean();
}

void MainWindow::createStatusBar()
{
	infoStatus = new KDCInfoStatusBar;
	statusBar()->insertWidget( 0, infoStatus, 13 );
	discSizeStatus = new KDCDiscSizeStatusBar;
	statusBar()->insertWidget( 1, discSizeStatus, 10 );
	discOptionsStatus = new KDCDiscOptionsStatusBar;
	connect( discOptionsStatus, SIGNAL( deleteFilesStatusChanged( bool ) ), this, SLOT( on_deleteFilesStatusChanged( bool ) ) );
	connect( discOptionsStatus, SIGNAL( deleteTempFilesStatusChanged( bool ) ), this, SLOT( on_deleteTempFilesStatusChanged( bool ) ) );
	connect( discOptionsStatus, SIGNAL( structureStatusChanged( bool ) ), this, SLOT( on_structureStatusChanged( bool ) ) );
	statusBar()->insertWidget( 2, discOptionsStatus, 1 );
	outputDirStatus = new KDCOutputDirStatusBar;
	statusBar()->insertWidget( 3, outputDirStatus, 4 );
	statusBar()->insertWidget( 4, new KDCLabel("ver. 0.3"), 1 );
}

void MainWindow::createActions()
{
	KStandardAction::preferences( this, SLOT( preferences() ), actionCollection() );
	quitAction = new KAction( KIcon("application-exit"), i18n("Quit"), this );
	quitAction->setShortcut( KStandardShortcut::shortcut( KStandardShortcut::Quit ) );
	connect( quitAction, SIGNAL( triggered() ), this, SLOT( on_quit() ) );

	addDiscAction = new KActionMenu( KIcon("media-optical"), i18n("New &Project.."), this );
	addDiscAction->setDelayed( FALSE );
	addDVDDiscAction = new KAction( KIcon("media-optical"), i18n("DVD Video Disc"), this );
	connect( addDVDDiscAction, SIGNAL( triggered() ), this, SLOT( on_addDVDDiscAction_triggered() ) );
	addCVDDiscAction = new KAction( KIcon("media-optical"), i18n("CVD Video Disc"), this );
	connect( addCVDDiscAction, SIGNAL( triggered() ), this, SLOT( on_addCVDDiscAction_triggered() ) );
	addSVCDDiscAction = new KAction( KIcon("media-optical"), i18n("SVCD Video Disc"), this );
	connect( addSVCDDiscAction, SIGNAL( triggered() ), this, SLOT( on_addSVCDDiscAction_triggered() ) );
	addVCDDiscAction = new KAction( KIcon("media-optical"), i18n("VCD Video Disc"), this );
	connect( addVCDDiscAction, SIGNAL( triggered() ), this, SLOT( on_addVCDDiscAction_triggered() ) );
	addBDDiscAction = new KAction( KIcon("media-optical"), i18n("Blu-ray Video Disc"), this );
	addHDDVDDiscAction = new KAction( KIcon("media-optical"), i18n("HD DVD Video Disc"), this );
	addDiscAction->addAction( addDVDDiscAction );
	addDiscAction->addAction( addCVDDiscAction );
	addDiscAction->addAction( addSVCDDiscAction );
	addDiscAction->addAction( addVCDDiscAction );
	addDiscAction->addAction( addHDDVDDiscAction );
	addDiscAction->addAction( addBDDiscAction );

	buildProjectAction = new KAction( KIcon("run-build-file"), i18n("&Build"), this );
	connect( buildProjectAction, SIGNAL( triggered() ), this, SLOT( on_buildProjectAction_triggered() ) );
	buildAllProjectsAction = new KAction( KIcon("run-build"), i18n("Build &All"), this );	
	connect( buildAllProjectsAction, SIGNAL( triggered() ), this, SLOT( on_buildAllProjectsAction_triggered() ) );

	addObjectAction = new KActionMenu( i18n("Add New Object..."), this );
	addVMGMenuAction = new KAction( KIcon("insert-image"), i18n("VMG &Menu"), this );
	connect( addVMGMenuAction, SIGNAL( triggered() ), this, SLOT( on_addVMGMenuAction_triggered() ) );
	addMovieAction = new KAction( KIcon("application-x-mplayer2"), i18n("M&ovie"), this );
	connect( addMovieAction, SIGNAL( triggered() ), this, SLOT( on_addMovieAction_triggered() ) );
	addVTSAction = new KAction( KIcon("folder-video"), i18n("&VTS"), this );
	connect( addVTSAction, SIGNAL( triggered() ), this, SLOT( on_addVTSAction_triggered() ) );
	addObjectAction->addAction( addVTSAction );
	addObjectAction->addAction( addVMGMenuAction );
	addObjectAction->addAction( addMovieAction );

	upAction = new KAction( KIcon("go-up"), i18n("Up"), this );
	upAction->setShortcut( QKeySequence( "PgUp" ) );
	connect( upAction, SIGNAL( triggered() ), this, SLOT( on_upAction_triggered() ) );
	downAction = new KAction( KIcon("go-down"), i18n("Down"), this );
	downAction->setShortcut( QKeySequence( "PgDown" ) );
	connect( downAction, SIGNAL( triggered() ), this, SLOT( on_downAction_triggered() ) );
	deleteAction = new KAction( KIcon("edit-delete"), i18n("Delete"), this );
	connect( deleteAction, SIGNAL( triggered() ), this, SLOT( on_deleteAction_triggered() ) );
	deleteAction->setShortcut( QKeySequence( "Del" ) );

	createGroupAction = new KActionMenu( i18n("Create Movie Clip with.."), this );
	addToGroupAction = new KActionMenu( i18n("Add to Movie Clip.."), this );
	removeFromGroupAction = new KActionMenu( i18n("Remove Movie Clip.."), this );
	previewAction = new KAction( KIcon("media-playback-start"), i18n("Preview"), this );
	connect( previewAction, SIGNAL( triggered() ), this, SLOT( on_previewAction_triggered() ) );
	destroyGroupAction = new KAction( i18n("Destroy Movie Clip"), this );
	connect( destroyGroupAction, SIGNAL( triggered() ), this, SLOT( on_destroyGroupAction_triggered() ) );

	dvdSchemeAction = new KActionMenu( i18n("DVD Structure"), this );
	separateVTSAction = new KAction( i18n("Each movie in separate VTS"), this );
	connect( separateVTSAction, SIGNAL( triggered() ), this, SLOT( on_separateVTSAction_triggered() ) );
	dvdSchemeAction->addAction( separateVTSAction );
}

void MainWindow::addActionsToCollection()
{
	actionCollection()->addAction( "quit", quitAction );
	actionCollection()->addAction( "addDisc", addDiscAction );
	actionCollection()->addAction( "addDVDDisc", addDVDDiscAction );
	actionCollection()->addAction( "addCVDDisc", addCVDDiscAction );
	actionCollection()->addAction( "addSVCDDisc", addSVCDDiscAction );
	actionCollection()->addAction( "addVCDDisc", addVCDDiscAction );
	actionCollection()->addAction( "addBDDisc", addBDDiscAction );
	actionCollection()->addAction( "addHDDVDDisc", addHDDVDDiscAction );
	actionCollection()->addAction( "addVMGMenu", addVMGMenuAction );
	actionCollection()->addAction( "addMovie", addMovieAction );
	actionCollection()->addAction( "addVTS", addVTSAction );
	actionCollection()->addAction( "up", upAction );
	actionCollection()->addAction( "down", downAction );
	actionCollection()->addAction( "delete", deleteAction );
	actionCollection()->addAction( "createGroup", createGroupAction );
	actionCollection()->addAction( "addToGroup", addToGroupAction );
	actionCollection()->addAction( "removeFromGroup", removeFromGroupAction );
	actionCollection()->addAction( "preview", previewAction );
	actionCollection()->addAction( "destroyGroup", destroyGroupAction );
	actionCollection()->addAction( "addObject", addObjectAction );
	actionCollection()->addAction( "build", buildProjectAction );
	actionCollection()->addAction( "buildAll", buildAllProjectsAction );
	actionCollection()->addAction( "dvdScheme", dvdSchemeAction );
}

void MainWindow::on_addMovieAction_triggered()
{
	KConfig config("kdvdcreator");
	KConfigGroup general( &config, "General" );
	QString dir = general.readEntry( "LastMovieDir", QDir::homePath() );
	QString filter( "*.avi *.mpg *.vob *.rmvb *.mkv *.wmv *.asf *.mp4 *.rm *.flc *.mov|All Video Files" );
	QStringList files = KFileDialog::getOpenFileNames(	KUrl( dir ), filter, this,
																											i18n("Select one or more video files to open") );
	if ( !files.isEmpty() )
	{
		general.writeEntry( "LastMovieDir", QFileInfo( files.first() ).path() );
		general.config()->sync();
		openFiles( files );
	}
}

void MainWindow::openFiles( const QStringList& files )
{
	if ( files.isEmpty() )
		return;
	qDebug( "MainWindow::openFiles()::start" );
	qDebug() << discFactory;
	KDCReadFileCompositeJob *read = new KDCReadFileCompositeJob( files, discFactory->currentDisc(), this );
	connect( read, SIGNAL( done( bool ) ), this, SLOT( on_readingFilesFinished( bool ) ) );
	setCursor( Qt::WaitCursor );
	read->start();
}

void MainWindow::on_readingFilesFinished( bool allMovies )
{
	if ( !allMovies )
		infoStatus->showMessage( i18n("Some media could not be loaded") );
	else
		infoStatus->showMessage( i18n("All files added sucesfully"), KDCInfoStatusBar::Info );
	setCursor( Qt::ArrowCursor );
	qDebug( "MainWindow::openFiles()::end" );
}

void MainWindow::on_addVMGMenuAction_triggered()
{
	//discFactory->currentDisc()->addVMGMenu();
	//addVMGMenuAction->setEnabled( FALSE );
}

void MainWindow::on_addDVDDiscAction_triggered()
{
	discFactory->addNewDisc( KDC::DVD );
	stateChanged( "dvd_disc_added" );
}

void MainWindow::on_addCVDDiscAction_triggered()
{
	discFactory->addNewDisc( KDC::CVD );
	stateChanged( "xvcd_disc_added" );
}

void MainWindow::on_addSVCDDiscAction_triggered()
{
	discFactory->addNewDisc( KDC::SVCD );
	stateChanged( "xvcd_disc_added" );
}

void MainWindow::on_addVCDDiscAction_triggered()
{
	discFactory->addNewDisc( KDC::VCD );
	stateChanged( "xvcd_disc_added" );
}

void MainWindow::on_upAction_triggered()
{
	discFactory->currentDisc()->moveUp();
}

void MainWindow::on_downAction_triggered()
{
	discFactory->currentDisc()->moveDown();
}

void MainWindow::on_deleteAction_triggered()
{
	qDebug( "MainWindow::deleteAction()::begin" );
	KDCDisc *disc = discFactory->currentDisc();
	disc->blockSignals( TRUE );
	QTreeWidgetItem *item = disc->currentItem();
	qDebug( "MainWindow::deleteAction()::begin:: item = 1, type = %d", item->type() );
	if ( item->type() == KDC::MovieInfo )
		item = item->parent();
	if ( item->type() == KDC::Movie )
		disc->deleteMovie( static_cast<KDCMovie*>( item ) ); 
	else
		if ( item->type() == KDC::MovieGroup )
			disc->deleteMovieGroup( static_cast<KDCMovieGroup*>( item ) );
	else
		if ( item->type() == KDC::VTS )
		{
			KDCDVDDisc *dvd = static_cast<KDCDVDDisc*>( disc );
			dvd->deleteVTS( item );
		}
	disc->blockSignals( FALSE );
	disc->refresh();
	qDebug( "MainWindow::deleteAction()::end" );
}

void MainWindow::preferences()
{
	KDCConfigDialog dlg;
	dlg.exec();
	if ( dlg.result() == QDialog::Accepted )
	{
		dlg.saveSettings();
		for ( int i=0; i<discFactory->count(); ++i )
			discFactory->getDisc( i )->setAnimated( KDCConfig::animation() );
	}
}

void MainWindow::on_addVTSAction_triggered()
{
	KDCDVDDisc *disc = static_cast<KDCDVDDisc*>( discFactory->currentDisc() );
	qDebug( "MainWindow::addVTSAction():: disc = %d", KDCLib::isNull( disc ) );
	if ( disc->vtsCount() < 99 )
		disc->addVTS();
	else
		infoStatus->showMessage( i18n("Max. number of VTS is 99") );
}

void MainWindow::on_previewAction_triggered()
{
	QTreeWidgetItem *movie = discFactory->currentDisc()->currentMovie();
	if ( movie )
	{
		qDebug( "MainWindow::previewAction()::movie->type() = %d", movie->type() );
		KDCStaticPreviewJob *job;
		if ( movie->type() == KDC::Movie )
			job = new KDCStaticPreviewJob( static_cast<KDCMovie*>( movie ) );
		else
			job = new KDCStaticPreviewJob( static_cast<KDCMovieGroup*>( movie ) );
		connect( job, SIGNAL( finished( KDCJob * ) ), this, SLOT( closePreview() ) );
		setEnabled( FALSE );
		job->start();
	}
}

void MainWindow::on_buildProjectAction_triggered()
{
	qDebug( "MainWindow::buildProjectAction()::begin" );
	QStringList errors = discFactory->currentDisc()->checkErrors();
	if ( errors.isEmpty() )
	{
		QList<KDCDisc*> list;
		list << discFactory->currentDisc();
		setVisible( FALSE );
		outputDirStatus->stop();
		KDCBuildAllCompositeJob *job = new KDCBuildAllCompositeJob( list );
		connect( job->dialog(), SIGNAL( finished() ), this, SLOT( on_processingDiscsFinished() ) );
		KDCOnFlyPreviewJob *prev = new KDCOnFlyPreviewJob( job->dialog() );
		connect( job, SIGNAL( done() ), prev, SLOT( deleteLater() ) );
		connect( job->dialog(), SIGNAL( closePreview() ), prev, SLOT( close() ) );
		connect( job->dialog(), SIGNAL( showPreview() ), prev, SLOT( show() ) );
		connect( job, SIGNAL( processingMovieChanged( QTreeWidgetItem * ) ), prev, SLOT( setCurrentMovie( QTreeWidgetItem * ) ) );
		connect( job, SIGNAL( progress( int ) ), prev, SLOT( setCurrentFrame( int ) ) );
		connect( prev, SIGNAL( done() ), job->dialog(), SLOT( uncheckPreview() ) );
		job->start();
	}
	else
		KMessageBox::errorList( this, i18n( "Current project has errors." ), errors );
	qDebug( "MainWindow::buildProjectAction()::end" );
}

void MainWindow::on_buildAllProjectsAction_triggered()
{
	qDebug( "MainWindow::buildAllProjectAction()::begin" );
	QStringList errors;
	QList<KDCDisc*> list;
	for ( int i=0; i<discFactory->count(); ++i )
	{
		errors << discFactory->getDisc( i )->checkErrors();
		list << discFactory->getDisc( i );
	}
	if ( errors.isEmpty() )
	{
		outputDirStatus->stop();
		setVisible( FALSE );
		KDCBuildAllCompositeJob *job = new KDCBuildAllCompositeJob( list );
		connect( job->dialog(), SIGNAL( finished() ), this, SLOT( on_processingDiscsFinished() ) );
		KDCOnFlyPreviewJob *prev = new KDCOnFlyPreviewJob( job->dialog() );
		connect( job, SIGNAL( done() ), prev, SLOT( deleteLater() ) );
		connect( job->dialog(), SIGNAL( closePreview() ), prev, SLOT( close() ) );
		connect( job->dialog(), SIGNAL( showPreview() ), prev, SLOT( show() ) );
		connect( job, SIGNAL( processingMovieChanged( QTreeWidgetItem * ) ), prev, SLOT( setCurrentMovie( QTreeWidgetItem * ) ) );
		connect( job, SIGNAL( progress( int ) ), prev, SLOT( setCurrentFrame( int ) ) );
		connect( prev, SIGNAL( done() ), job->dialog(), SLOT( uncheckPreview() ) );
		job->start();
	}
	else
		KMessageBox::errorList( this, i18n( "Projects have errors." ), errors );
	qDebug( "MainWindow::buildAllProjectAction()::end" );
}

void MainWindow::on_destroyGroupAction_triggered()
{
	QTreeWidgetItem *item = discFactory->currentDisc()->currentItem();
	qDebug( "MainWindow::destroyGroupAction()::begin::type = %d", item->type() );
	if ( item->type() == KDC::MovieInfo )
		item = item->parent();
	KDCDisc *disc = discFactory->currentDisc();
	disc->blockSignals( TRUE );
	KDCMovieGroup *group = static_cast<KDCMovieGroup*>( item );
	disc->destroyMovieGroup( group );
	disc->blockSignals( FALSE );
	disc->refresh();
	qDebug( "MainWindow::destroyGroupAction()::end" );
}

void MainWindow::on_createGroupAction_triggered()
{
	qDebug( "MainWindow::createGroupAction()::begin" );
	KAction *action = qobject_cast<KAction*>( sender() );
	if ( action )
	{
		KDCDisc *disc = discFactory->currentDisc();
		disc->blockSignals( TRUE );
		qDebug( "MainWindow::createGroupAction()::currentDisc = %d, type = %d", KDCLib::isNull( disc ), disc->discType() );
		qDebug( "MainWindow::createGroupAction()::movie,type = %d, item = %d", disc->currentMovie()->type(), disc->currentItem()->type() );
		KDCMovie *movie = static_cast<KDCMovie*>( disc->currentMovie() );
		KDCMovieGroup *group = new KDCMovieGroup;
		group->setOutputName( KDCFileFactory::baseFileName( movie->fileName() ) );
		if ( disc->discType() == KDC::DVD )
			disc->insertMovieGroup( disc->currentItem()->parent()->indexOfChild( movie ), group );
		else
			disc->insertMovieGroup( disc->indexOfTopLevelItem( disc->currentItem() ), group );
		qDebug( "MainWindow::createGroupAction()::currentMovie = %d", KDCLib::isNull( movie ) );

		disc->moveMovieToMovieGroup( movie, group );
		movie = disc->movie( action->data().toInt() );
		disc->moveMovieToMovieGroup( movie, group );
		disc->blockSignals( FALSE );
		disc->setCurrentItem( group->child( 0 ) );
		disc->refresh();
	}
	qDebug( "MainWindow::createGroupAction()::end" );
}

void MainWindow::on_addToGroupAction_triggered()
{
	KAction *action = qobject_cast<KAction*>( sender() );
	if ( action )
	{
		KDCDisc *disc = discFactory->currentDisc();
		disc->blockSignals( TRUE );
		KDCMovie *movie = static_cast<KDCMovie*>( disc->currentMovie() );
		KDCMovieGroup *group = disc->movieGroup( action->data().toInt() );
		disc->moveMovieToMovieGroup( movie, group );
		disc->setCurrentItem( group );
		disc->blockSignals( FALSE );
		disc->refresh();
	}
}

void MainWindow::on_removeFromGroupAction_triggered()
{
	qDebug( "MainWindow::removeFromGroupAction()::begin" );
	KAction *action = qobject_cast<KAction*>( sender() );
	if ( action )
	{
		KDCDisc *disc = discFactory->currentDisc();
		disc->blockSignals( TRUE );
		KDCMovieGroup *group = static_cast<KDCMovieGroup*>( disc->currentItem() );
		KDCMovie *movie = group->takeMovie( action->data().toInt() );
		if ( disc->discType() == KDC::DVD )
		{
			QTreeWidgetItem *vts = disc->currentItem()->parent();
			int index = vts->indexOfChild( disc->currentItem() );
			if ( group->count() == 1 )
				disc->destroyMovieGroup( group );
			vts->insertChild( index, movie );
		}
		else
		{
			int index = disc->indexOfTopLevelItem( disc->currentItem() );
			if ( group->count() == 1 )
				disc->destroyMovieGroup( group );
			disc->insertTopLevelItem( index, movie );
		}
		disc->blockSignals( FALSE );
		disc->refresh();
	}
	qDebug( "MainWindow::removeFromGroupAction()::end" );
}

void MainWindow::preparePopupMenu()
{
	qDebug( "MainWindow::preparePopupMenu()::begin" );
	//if disc structure is empty
	KDCDisc *disc = discFactory->currentDisc();
	QTreeWidgetItem *item = disc->currentItem();
	qDebug( "MainWindow::preparePopupMenu():: item = %d", KDCLib::isNull( item ) );
	if ( !item )
	{
		stateChanged( "all_disable_popup" );
		return;
	}

	if ( item->type() == KDC::Movie )
	{
		KDCMovie *movie = static_cast<KDCMovie*>( item );
		// create movie group
		removeFromGroupAction->menu()->clear();
		createGroupAction->menu()->clear();
		movieActions.clear();

		QStringList files = disc->codeMovies( movie );

		QStringList tmp;
		int i;
		for ( i=0; i<files.size(); ++i )
		{
			movieActions.append( new KAction( 0 ) );
			tmp = files.at( i ).split( "\t" );
			movieActions.last()->setText( tmp.at( 0 ) );
			movieActions.last()->setData( tmp.at( 1 ).toInt() );
			connect( movieActions.at( i ), SIGNAL( triggered() ), this, SLOT( on_createGroupAction_triggered() ) );
			createGroupAction->addAction( movieActions.last() );
		}
		createGroupAction->setEnabled( movieActions.size() > 0 );

		// add current movie to movie group
		addToGroupAction->menu()->clear();
		groupActions.clear();

		files = disc->codeMovieGroups( movie );
		for ( i=0; i<files.size(); ++i )
		{
			groupActions.append( new KAction( 0 ) );
			tmp = files.at( i ).split( "\t" );
			groupActions.last()->setText( tmp.at( 0 ) );
			groupActions.last()->setData( tmp.at( 1 ).toInt() );
			connect( groupActions.at( i ), SIGNAL( triggered() ), this, SLOT( on_addToGroupAction_triggered() ) );
			addToGroupAction->addAction( groupActions.last() );
		}
		addToGroupAction->setEnabled( groupActions.size() > 0 );
	}
	else
		if ( item->type() == KDC::MovieGroup )
		{
			KDCMovieGroup *group = static_cast<KDCMovieGroup*>( item );
			createGroupAction->menu()->clear();
			removeFromGroupAction->menu()->clear();
			movieActions.clear();
			
			QStringList files = group->codeMovies();
			QStringList tmp;
			for ( int i=0; i<files.size(); ++i )
			{
				movieActions.append( new KAction( 0 ) );
				tmp = files.at( i ).split( "\t" );
				movieActions.last()->setText( tmp.at( 0 ) );
				movieActions.last()->setData( tmp.at( 1 ).toInt() );
				connect( movieActions.at( i ), SIGNAL( triggered() ), this, SLOT( on_removeFromGroupAction_triggered() ) );
				removeFromGroupAction->addAction( movieActions.last() );
			}
			removeFromGroupAction->setEnabled( movieActions.size() > 0 );
		}
	qDebug( "MainWindow::preparePopupMenu()::end" );
}

void MainWindow::on_currentObjectChanged( QTreeWidgetItem *item )
{
	qDebug( "MainWindow::currentObjectChanged( %d )::begin", KDCLib::isNull( item ) );
	if ( !item )
		stateChanged( "all_disable_popup" );
	else
	{
		if ( item->type() == KDC::VMGMenu )
			stateChanged( "menu_selection" );
		else
			if ( item->type() == KDC::VTS )
				stateChanged( "vts_selection" );
		else
			if ( item->type() == KDC::MovieGroup )
				stateChanged( "group_selection" );
		else
			stateChanged( "movie_selection" );
	}
	preparePopupMenu();
	qDebug( "MainWindow::currentObjectChanged()::end" );
}

void MainWindow::on_canMoveUpDown( bool canUp, bool canDown )
{
	qDebug( "MainWindow::canMoveUpDown()" );
	upAction->setEnabled( canUp );
	downAction->setEnabled( canDown );
}

void MainWindow::on_quit()
{
	// TODO
	close();
}

void MainWindow::on_currentDiscChanged( KDCDisc *disc )
{
	qDebug( "MainWindow::currentDiscChanged( %d )::begin", KDCLib::isNull( disc ) );
	if ( disc )
	{
		if ( disc->discType() == KDC::DVD )
			stateChanged( "dvd" );
		else
			stateChanged( "dvd", StateReverse );
		on_currentDiscSettingsChanged( disc->settings()->mediaSize(), disc->size(), disc->settings()->outputPath() );
		discOptionsStatus->setDeleteFilesEnabled( disc->settings()->deleteFilesEnabled() );
		discOptionsStatus->setDeleteTempFilesEnabled( disc->settings()->deleteTempFilesEnabled() );
		discOptionsStatus->setStructureEnabled( disc->settings()->structureEnabled() );
		discOptionsStatus->setSignalsEnabled( TRUE );
	}
	else
	{
		stateChanged( "apps_clear" );
		outputDirStatus->reset();
		discSizeStatus->reset();
		discOptionsStatus->reset();
		discOptionsStatus->setSignalsEnabled( FALSE );
	}
	qDebug( "MainWindow::currentDiscChanged()::end" );
}

void MainWindow::on_currentDiscSettingsChanged( KDC::MediaSize mediaSize, quint64 newSize, const QString& path )
{
	qDebug( "MainWindow::currentDiscSizeChanged()" );
	discSizeStatus->updateInfo( mediaSize, newSize );
	outputDirStatus->setOutputPath( path );
	outputDirStatus->setProjectSize( newSize );
}

void MainWindow::on_currentDiscContextMenu( const QPoint& pos )
{
	qDebug( "MainWindow::on_currentDiscContextMenu::begin" );
	KMenu *popupMenu = (KMenu*)factory()->container("discPopupMenu", this );
	KMenu *tmp = discFactory->currentDiscMenu();
	tmp->addAction( discOptionsStatus->deleteFilesAction() );
	tmp->addAction( discOptionsStatus->deleteTempFilesAction() );
	tmp->addAction( discOptionsStatus->discStructureAction() );
	if ( !popupMenu )
	  popupMenu = new KMenu( this );
	QAction *action = popupMenu->addMenu( tmp );
	popupMenu->exec( pos );
	popupMenu->removeAction( action );
	tmp->removeAction( discOptionsStatus->deleteFilesAction() );
	tmp->removeAction( discOptionsStatus->deleteTempFilesAction() );
	tmp->removeAction( discOptionsStatus->discStructureAction() );
	qDebug( "MainWindow::on_currentDiscContextMenu::end" );
}

void MainWindow::on_needToCloseDisc( KDCDisc *disc )
{
	int res = KMessageBox::questionYesNo( 0, i18n("Are you sure?") );
	if ( res == KMessageBox::Yes )
		discFactory->closeDisc( disc );
}

void MainWindow::on_processingDiscsFinished()
{
	outputDirStatus->start();
	setVisible( TRUE );
}

void MainWindow::closePreview()
{
	setEnabled( TRUE );
}
void MainWindow::on_separateVTSAction_triggered()
{
	KDCDVDDisc *disc = static_cast<KDCDVDDisc*>( discFactory->currentDisc() );
	disc->allMovieInSeparateVTS();
}

void MainWindow::on_deleteFilesStatusChanged( bool enabled )
{
	discFactory->currentDisc()->settings()->setDeleteFilesEnabled( enabled );
}

void MainWindow::on_deleteTempFilesStatusChanged( bool enabled )
{
	discFactory->currentDisc()->settings()->setDeleteTempFilesEnabled( enabled );
}

void MainWindow::on_structureStatusChanged( bool enabled )
{
	discFactory->currentDisc()->settings()->setStructureEnabled( enabled );
}
