/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcstreammodecombobox.h"

#include "../../kdcdisc.h"

#include <QVariant>
#include <QList>

KDCStreamModeComboBox::KDCStreamModeComboBox(	const KDCMovie *movie, KDCStreamModeComboBox::Mode mode,
																							int id, QWidget *parent ) : KComboBox( parent )
{
	KDC::DiscType type = movie->disc()->discType();
	KDC::StreamMode streamMode;
	int i;
	QList<bool> copy;
	if ( mode == Video )
	{
		streamMode = movie->videoStream()->mode();
		QList<KDCVideoStreamMetaData*> metaData;
		if ( movie->movieGroup() )
		{
			KDCMovieGroup *group = movie->movieGroup();
			for ( i=0; i<group->count(); ++i )
				metaData.append( group->movie( i )->videoStream()->metaData() );
		}
		else
			metaData.append( movie->videoStream()->metaData() );
		if ( type == KDC::DVD )
		{
			for ( i=0; i<metaData.size(); ++i )
				copy.append( KDCVideoStream::isDVDPALVideoStream( metaData.at( i ) ) || KDCVideoStream::isDVDNTSCVideoStream( metaData.at( i ) ) );
		}
		else
			if ( type == KDC::CVD )
			{
				for ( i=0; i<metaData.size(); ++i )
					copy.append( KDCVideoStream::isCVDPALVideoStream( metaData.at( i ) ) || KDCVideoStream::isCVDNTSCVideoStream( metaData.at( i ) ) );
			}
		else
			if ( type == KDC::SVCD )
			{
				for ( i=0; i<metaData.size(); ++i )
					copy.append( KDCVideoStream::isSVCDPALVideoStream( metaData.at( i ) ) || KDCVideoStream::isSVCDNTSCVideoStream( metaData.at( i ) ) );
			}
		else
			if ( type == KDC::VCD )
			{
				for ( i=0; i<metaData.size(); ++i )
					copy.append( KDCVideoStream::isVCDPALVideoStream( metaData.at( i ) ) || KDCVideoStream::isVCDNTSCVideoStream( metaData.at( i ) ) );
			}
		copy.removeAll( TRUE );
		if ( copy.isEmpty() )
			addItem( "Copy", KDC::Copy );
	}
	else
	{
		streamMode = movie->audioStreamGroup()->stream( id )->mode();
		QList<KDCAudioStreamMetaData*> metaData;
		if ( movie->movieGroup() )
		{
			KDCMovieGroup *group = movie->movieGroup();
			for ( i=0; i<group->count(); ++i )
				metaData.append( group->movie( i )->audioStreamGroup()->stream( id )->metaData() );
		}
		else
			metaData.append( movie->audioStreamGroup()->stream( id )->metaData() );
		if ( type == KDC::DVD )
		{
			for ( i=0; i<metaData.size(); ++i )
				if ( movie->videoFormat() == KDC::PAL )
					copy.append( KDCAudioStream::isDVDPALAudioStream( metaData.at( i ) ) );
				else
					copy.append( KDCAudioStream::isDVDNTSCAudioStream( metaData.at( i ) ) );
		}
		else
			if ( type == KDC::CVD )
			{
				for ( i=0; i<metaData.size(); ++i )
					copy.append( KDCAudioStream::isCVDAudioStream( metaData.at( i ) ) );
			}
		else
			if ( type == KDC::SVCD )
			{
				for ( i=0; i<metaData.size(); ++i )
					copy.append( KDCAudioStream::isSVCDAudioStream( metaData.at( i ) ) );
			}
		else
			if ( type == KDC::VCD )
			{
				for ( i=0; i<metaData.size(); ++i )
					copy.append( KDCAudioStream::isVCDAudioStream( metaData.at( i ) ) );
			}
		copy.removeAll( TRUE );
		if ( copy.isEmpty() )
			addItem( "Copy", KDC::Copy );
	}
	
	addItem( "Resample", KDC::Resample );
	setCurrentIndex( findData( streamMode ) );
	connect( this, SIGNAL( currentIndexChanged( int ) ), this, SLOT( on_currentIndexChanged( int ) ) );
}

KDCStreamModeComboBox::~KDCStreamModeComboBox()
{

}

void KDCStreamModeComboBox::setCurrentMode( KDC::StreamMode mode )
{
	setCurrentIndex( findData( mode ) );
}

KDC::StreamMode KDCStreamModeComboBox::currentMode()
{
	if ( currentText() == "Copy" )
		return KDC::Copy;
	else
		return KDC::Resample;
}

void KDCStreamModeComboBox::on_currentIndexChanged( int index )
{
	emit currentModeChanged( currentMode() );
}
