/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcmediaplayer.h"

#include "../kdcfilefactory.h"
#include "../lib/kdclib.h"

#include <QVBoxLayout>
#include <QList>
#include <QStringList>

#include <KLocale>
#include <KMessageBox>

KDCMediaPlayer::KDCMediaPlayer( QWidget *parent )
	: QWidget( parent )
{
	p_flag = FALSE;
	p_isPlaying = FALSE;

	setMinimumWidth( 285 );
	setMaximumWidth( 285 );

	p_mainWidget = new KDCMPlayerWidget;
	p_mainWidget->setWindowSize( 280, 158 );

	QVBoxLayout *mainLayout = new QVBoxLayout( this );
	mainLayout->addWidget( p_mainWidget );
	mainLayout->addStretch();

	p_mJob = new KDCMPlayerJob;
	p_mJob->setWindowID( p_mainWidget->windowID() );
	connect( p_mainWidget, SIGNAL( playClicked() ), this, SLOT( playCurrentMovie() ) );
	connect( p_mainWidget, SIGNAL( pauseClicked() ), p_mJob, SLOT( forcePause() ) );
	connect( p_mainWidget, SIGNAL( pauseClicked() ), this, SLOT( setPause() ) );
	connect( p_mainWidget, SIGNAL( stopClicked() ), p_mJob, SLOT( stop() ) );
	connect( p_mJob, SIGNAL( progress( int ) ), this, SLOT( mplayerProgress( int ) ) );
	connect( p_mainWidget, SIGNAL( timeSeekChanged( int ) ), this, SLOT( seekChanged( int ) ) );
	connect( p_mJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
	connect( p_mainWidget, SIGNAL( audioIDChanged( int ) ), p_mJob, SLOT( setAudioID( int ) ) );
	connect( p_mainWidget, SIGNAL( videoDriverChanged( const QString& ) ), p_mJob, SLOT( setVideoDriver( const QString& ) ) );
}

KDCMediaPlayer::~KDCMediaPlayer()
{
	if ( p_mJob )
		delete p_mJob;
}

void KDCMediaPlayer::playCurrentMovie()
{
	if ( ( !p_movie && !p_group ) || !p_flag )
		return;

	p_mJob->stop();
	KDCVideoStreamMetaData *metaData;
	int length = 0;
	if ( p_movie )
	{
		metaData = p_movie->videoStream()->metaData();
		length = qRound( metaData->length()/1000.0 );
		p_mJob->setFile( p_movie->fileName() );
	}
	else
	{
		metaData = p_group->movie( 0 )->videoStream()->metaData();
		QStringList files;
		KDCMovie *tmp;
		for ( int i=0; i<p_group->count(); ++i )
		{
			tmp = p_group->movie( i );
			length += tmp->videoStream()->metaData()->length();
			files << tmp->fileName();
		}
		length = qRound( length/1000.0 );
		p_mJob->setFiles( files );
	}
	p_mainWidget->setMovieProperties( metaData->width(), metaData->height(), metaData->aspectRatio(), length );
	qDebug( "KDCMediaPlayer::playCurrentMovie()::length = %d", length );
	p_mJob->setVideoDriver( p_mainWidget->videoDriver() );
	p_mJob->setAudioID( p_mainWidget->audioID() );
	p_mJob->start();
	p_flag = FALSE;
	p_isPlaying = TRUE;
}

void KDCMediaPlayer::setCurrentMovie( KDCMovie *movie )
{
	if ( movie && movie != p_movie )
	{
		clean();
		p_movie = movie;
		p_mainWidget->setAudioTracks( getAudioTracks( movie ) );
		p_flag = TRUE;
	}
}

void KDCMediaPlayer::setCurrentMovie( KDCMovieGroup *group )
{
	if ( group && group != p_group )
	{
		clean();
		p_group = group;
		p_mainWidget->setAudioTracks( getAudioTracks( p_group->movie( 0 ) ) );
		p_flag = TRUE;
	}
}

void KDCMediaPlayer::clean()
{
	p_movie = NULL;
	p_group = NULL;
	p_mJob->stop();
	p_mainWidget->setTimeSeek( 0 );
	p_flag = FALSE;
	p_isPlaying = FALSE;
}

void KDCMediaPlayer::on_jobFinished( KDCJob *job )
{
	qDebug( "KDCMediaPlayer::finished()" );
	if ( job->hasErrors() )
		KMessageBox::error( this,	i18n("KDC MediaPlayer fails with error message:\n<b>") +
															job->error() + "</b>" );
	p_flag = TRUE;
	p_isPlaying = FALSE;
}

QList<int> KDCMediaPlayer::getAudioTracks( const KDCMovie *movie )
{
	QList<int> list;

	if ( movie->audioStreamGroup() )
		for ( int i=0; i<movie->audioStreamGroup()->count(); ++i )
			list.append( movie->audioStreamGroup()->stream( i )->metaData()->id() );
	return list;
}

KDCMPlayerWidget* KDCMediaPlayer::playerWidget() const
{
	return p_mainWidget;
}

void KDCMediaPlayer::mplayerProgress( int value )
{
	if ( p_group )
	{
		qDebug( "Value = %d, Pos = %d, id = %d", value, p_mainWidget->currentPos(), p_mJob->currentMovie() );
		int id = p_mJob->currentMovie();
		if ( value == 0 && p_mainWidget->currentPos() > 0 )
		{
			++id;
			p_mJob->setCurrentMovie( id );
		}
		if ( id > 1 )
		{
			qDebug( "KDCMediaPlayer::mplayerProgress( %d )::id = %d", value, id );
			int ss = 0;
			for ( int i=0; i<id-1; ++i )
				ss += p_group->movie( i )->videoStream()->metaData()->length();
			value += qRound( ss/1000.0 );
			qDebug( "KDCMediaPlayer::mplayerProgress()::newValue = %d", value );
		}
	}
	p_mainWidget->setTimeSeek( value );
}

void KDCMediaPlayer::seekChanged( int value )
{
	if ( p_group )
	{
		int newSum = 0;
		int oldSum = 0;
		int i;
		for ( i=0; i<p_group->count(); ++i )
		{
			oldSum = newSum;
			newSum += qRound( p_group->movie( i )->videoStream()->metaData()->length()/1000.0 );
			if ( newSum >= value )
				break;
		}
		int id = p_mJob->currentMovie();
		int delta = i + 1 - id;
		if ( delta != 0 )
		{
			p_mJob->blockSignals( TRUE );
			if ( delta > 0 )
				p_mJob->playNext( delta );
			else
				if ( delta < 0 )
					p_mJob->playPrev( qAbs( delta ) );
			p_mJob->blockSignals( FALSE );
		}
		value -= oldSum;
		p_mJob->setCurrentMovie( id + delta );
		qDebug( "KDCMediaPlayer::seekChanged()::newValue = %d, id = %d, newID = %d", value, id, id + delta );
	}
	if ( p_mJob->isPause() )
	{
		p_mJob->forcePause();
		p_mJob->setSeek( value );
		p_mJob->forcePause();
	}
	else
		p_mJob->setSeek( value );
}

void KDCMediaPlayer::hideEvent( QHideEvent *event )
{
	QWidget::hideEvent( event );
	if ( p_isPlaying )
		p_mJob->forcePause();
}

void KDCMediaPlayer::showEvent( QShowEvent *event )
{
	QWidget::showEvent( event );
	if ( p_isPlaying )
		p_mJob->forcePause();
}

void KDCMediaPlayer::setPause()
{
	p_isPlaying = !p_isPlaying;
}
