/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcmovieinfowidget.h"

#include "../lib/kdclib.h"
#include "../kdcdisc.h"

#include <QStringList>
#include <QTime>
#include <QTreeWidgetItem>
#include <QHeaderView>
#include <QTreeWidgetItemIterator>

#include <KLocale>
#include <KIcon>

KDCMovieInfoWidget::KDCMovieInfoWidget( KDCMovie *movie, QWidget *parent )
	: QTreeWidget( parent )
{
	header()->hide();
	setColumnCount( 2 );
	setRootIsDecorated( FALSE );
	setIconSize( QSize( 24, 24 ) );

	setMinimumWidth( 275 );
	setMaximumWidth( 275 );
	setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );

	setColumnWidth( 0, 125 );
	setColumnWidth( 1, 120 );

	p_delegate = new KDCMovieInfoItemDelegate;
	setItemDelegate( p_delegate );

	QTreeWidgetItem *stream;
	QTreeWidgetItem *item;
	//video
	KDCVideoStreamMetaData *video = movie->videoStream()->metaData();
	stream = new QTreeWidgetItem;
	stream->setText( 0, i18n("\tStream 1") );
	stream->setIcon( 0, KIcon("application-x-mplayer2") );
	new QTreeWidgetItem( stream, QStringList() << i18n("\tType:") << "Video" );
	new QTreeWidgetItem( stream, QStringList() << i18n("\tDemuxer:") << video->demuxer() );
	new QTreeWidgetItem( stream, QStringList() << i18n("\tFormat:") << KDCLib::videoCodecName( video->codec() ) );
	new QTreeWidgetItem( stream, QStringList() << i18n("\tWidth:") << QString::number( video->width() ) );
	new QTreeWidgetItem( stream, QStringList() << i18n("\tHeight:") << QString::number( video->height() ) );
	new QTreeWidgetItem( stream, QStringList() << i18n("\tFps:") << QString::number( video->fps() ) );
	new QTreeWidgetItem( stream, QStringList() << i18n("\tBitrate:") << QString::number( video->bitrate() ) + " kbps" );
	new QTreeWidgetItem( stream, QStringList() << i18n("\tAspect:") << QString::number( video->aspectRatio() ) );
	addTopLevelItem( stream );
	stream->setExpanded( TRUE );

	stream = new QTreeWidgetItem;
	stream->setText( 0, i18n("\tGeneral") );
	QTime time;
	item = new QTreeWidgetItem( stream, QStringList() << i18n("\tLength:") << time.addMSecs( video->length() ).toString("hh:mm:ss.zzz") );
	item->setIcon( 0, KIcon("player-time") );
	addTopLevelItem( stream );
	stream->setExpanded( TRUE );

	//audio
	KDCAudioStreamMetaData *audio;
	if ( movie->audioStreamGroup() )
	{
		for ( int i=0; i<movie->audioStreamGroup()->count(); ++i )
		{
			audio = movie->audioStreamGroup()->stream( i )->metaData();
			stream = new QTreeWidgetItem;
			stream->setText( 0, i18n("\tStream ") + QString::number( i + 2 ) );
			stream->setIcon( 0, KIcon("audio-basic") );
			new QTreeWidgetItem( stream, QStringList() << i18n("\tType:") << "Audio" );
			new QTreeWidgetItem( stream, QStringList() << i18n("\tFormat:") << KDCLib::audioCodecName( audio->codec() ) );
			new QTreeWidgetItem( stream, QStringList() << i18n("\tChannels:") << QString::number( audio->channelCount() ) );
			new QTreeWidgetItem( stream, QStringList() << i18n("\tBitrate:") << QString::number( audio->bitrate() ) + " kbps" );
			new QTreeWidgetItem( stream, QStringList() << i18n("\tSample Rate:") << QString::number( audio->sampleRate() ) + " Hz" );
			addTopLevelItem( stream );
			stream->setExpanded( TRUE );
		}
	}

	QTreeWidgetItemIterator it( this );
	while( *it )
	{
		( *it )->setFlags( Qt::ItemIsEnabled );
		++it;
	}
}

KDCMovieInfoWidget::~KDCMovieInfoWidget()
{
	if ( p_delegate )
		delete p_delegate;
}

void KDCMovieInfoWidget::drawBranches( QPainter *painter, const QRect& rect, const QModelIndex& index ) const
{

}
